try:
    from .coin import Coin
    from .gold import Gold
    from .graph import Graph
    from .currency import Currency
except:  # noqa
    from coin import Coin
    from gold import Gold
    from graph import Graph
    from currency import Currency



import datetime


class Token:
    def __init__(self, value: str, life_span: int = 600, generated_at: datetime.datetime = datetime.datetime.now()):
        """
        :param value: token value
        :param life_span: token life span in seconds
        :param generated_at: token generation date
        """
        self.life_span = life_span
        self.generated_at = generated_at
        self.value = value

    def is_expired(self) -> bool:
        """
        :return: True if token is expired
        """
        return datetime.datetime.now() - self.generated_at > datetime.timedelta(seconds=self.life_span)



import datetime
from typing import Optional

try:
    from ..models.token import *
    from .storage_manager import *
    from ..helpers.utils import Singleton
    from ..server import get_token_from_main_page
except:  # noqa
    from src.bonbast.models.token import *
    from src.bonbast.managers.storage_manager import *
    from src.bonbast.helpers.utils import Singleton
    from src.bonbast.server import get_token_from_main_page


class TokenManager(object, metaclass=Singleton):
    _storage_manager = StorageManager(file_path=StorageManager.get_app_directory() / 'token.data')

    @staticmethod
    def _delete_token():
        TokenManager._storage_manager.delete_file()

    @staticmethod
    def _save_token(token: Token) -> None:
        TokenManager._storage_manager.save_file(f'{token.value}\n{token.generated_at.isoformat()}')

    @staticmethod
    def _load_token() -> Optional[Token]:
        try:
            token, date = TokenManager._storage_manager.load_file().splitlines()
            return Token(value=token, generated_at=datetime.datetime.fromisoformat(date))
        except FileNotFoundError:
            return None

    def generate(self):
        """
        :return: a new [fresh] Token instance
        """
        token = self._load_token()
        if token is None or token.is_expired():
            token = Token(get_token_from_main_page())
            self._save_token(token)

        return token

    def invalidate_token(self):
        self._delete_token()


token_manager = TokenManager()

__all__ = [
    'token_manager',
]



import math

from typing import List
from rich.table import Table
from rich.console import ConsoleRenderable

try:
    from .models import *
except ImportError:
    from models import *


def __get_currencies_sub_table(currencies: List[Currency]) -> ConsoleRenderable:
    table = Table()

    table.add_column("Code", style="cyan", no_wrap=True)
    table.add_column("Currency", style="cyan")
    table.add_column("Sell", no_wrap=True)
    table.add_column("Buy", no_wrap=True)

    for currency in currencies:
        table.add_row(
            currency.code,
            currency.name,
            currency.formatted_sell if currency.sell is not None else '-',
            currency.formatted_buy if currency.buy is not None else '-',
        )

    return table


def get_currencies_table(currencies: List[Currency], columns: int) -> ConsoleRenderable:
    """ Gets a list of data.Currency and generates currencies table
    """

    filtered_currencies = [
        currency for currency in currencies if
        (currency.sell is not None or currency.buy is not None) and (currency.sell != 0 or currency.buy != 0)
    ]

    if len(filtered_currencies) < 6:
        columns = 1

    each_part_count = math.ceil(len(filtered_currencies) / columns)
    currencies_parts = [
        filtered_currencies[i:i + each_part_count] for i in
        range(0, len(filtered_currencies), each_part_count)
    ]
    tables = [__get_currencies_sub_table(part) for part in currencies_parts]

    grid = Table.grid(padding=5)
    grid.title = 'Currencies'
    for _ in range(columns):
        grid.add_column()
    grid.add_row(*tables)

    return grid


def get_coins_table(coins: List[Coin]) -> ConsoleRenderable:
    """ Gets a list of data.Coin and generates coins table
    """
    table = Table(title="Coins")

    table.add_column("Coin", style="cyan")
    table.add_column("Sell", no_wrap=True)
    table.add_column("Buy", no_wrap=True)

    for coin in coins:
        table.add_row(
            coin.name,
            coin.formatted_sell if coin.sell is not None else '-',
            coin.formatted_buy if coin.buy is not None else '-',
        )

    return table


# get gold and it's price in a table
def get_gold_table(golds: List[Gold]) -> ConsoleRenderable:
    """ Gets a list of data.Gold and generates golds table
    """
    table = Table(title="Gold")

    table.add_column("Gold", style="cyan")
    table.add_column("Price", no_wrap=True)

    for gold in golds:
        table.add_row(
            gold.name,
            gold.formatted_price if gold.price is not None else '-',
        )

    return table



from __future__ import annotations

from rich.text import Text

try:
    from ..helpers.utils import *
except ImportError:
    from src.bonbast.helpers.utils import *


class Coin:
    """ Coin model
    """
    VALUES = {
        'emami1': 'Emami',
        'azadi1g': 'Gerami',
        'azadi1': 'Azadi',
        'azadi1_2': '½ Azadi',
        'azadi1_4': '¼ Azadi',
    }

    def __init__(self, code: str, name: str, sell: int, buy: int):
        self.code = code
        self.name = name
        self.sell = sell
        self.buy = buy

    @property
    def formatted_buy(self) -> str:
        return format_toman(self.buy)

    @property
    def formatted_sell(self) -> str:
        return format_toman(self.sell)

    def to_json(self) -> dict:
        return {
            self.code: {
                'name': self.name,
                'sell': self.sell,
                'buy': self.buy,
            }
        }

    def assemble_simple_text(self, old_coin: Coin, **kwargs) -> Text:
        return Text.assemble(
            f'{self.name}: ',
            (f'{self.formatted_sell}', get_color(self.sell, old_coin.sell) if old_coin is not None else ''),
            ' / ',
            (f'{self.formatted_buy}', get_color(self.buy, old_coin.buy) if old_coin is not None else ''),
            '\n'
        )



import json

from rich.console import Console
from rich.live import Live
from rich.pretty import pprint
from rich.text import Text

try:
    from .models import *
    from .server import *
    from .tables import *
    from .managers.token_manager import *
    from .managers.storage_manager import *
    from .helpers.click_callbacks import *
    from .helpers.utils import *
except ImportError:
    from models import *
    from server import *
    from tables import *
    from managers.token_manager import *
    from managers.storage_manager import *
    from helpers.click_callbacks import *
    from helpers.utils import *


@retry(message='Error: token is expired. Try again later.')
def get_prices(show_only: List[str] = None):
    token = token_manager.generate()
    try:
        response = get_prices_from_api(token.value)
        if show_only is not None and len(show_only) > 0:
            response = list(response)
            for index, item in enumerate(response):
                response[index] = [item for item in item if item.code.lower() in show_only]
            response = tuple(response)

        return response
    except ResetAPIError as e:
        token_manager.invalidate_token()
        raise e


# ================ bonbast ================
@click.group(invoke_without_command=True)
@click.option('-v', '--version', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True)
@click.option(
    '--show-only',
    help='Show only specified currencies, coins, or golds (separated by comma)',
    callback=parse_show_only
)
@click.pass_context
def cli(ctx, show_only):
    if ctx.invoked_subcommand is None:
        currencies_list, coins_list, golds_list = get_prices(show_only)
        console = Console()

        if currencies_list is not None and len(currencies_list) > 0:
            console.print(get_currencies_table(currencies_list, columns=2))

        if coins_list is not None and len(coins_list) > 0:
            console.print(get_coins_table(coins_list))

        if golds_list is not None and len(golds_list) > 0:
            console.print(get_gold_table(golds_list))


# @cli.command()
# def graph():
#     click_helper.echo('Draw graph without live update')


# ================ bonbast live ================
@cli.group(invoke_without_command=True)
@click.pass_context
def live(ctx):
    if ctx.invoked_subcommand is None:
        print('Show full table with live prices / up and down arrows')
    pass


# ================ bonbast live graph ================
# @live.command('graph')
# def live_graph():
#     print('show graph updating live every x seconds')
#     pass


# ================ bonbast live simple ================
@live.command('simple')
@click.option('-i', '--interval', type=click.IntRange(min=1), default=30, help='Interval in seconds')
@click.option(
    '--show-only',
    help='Show only specified currencies, coins, or golds (separated by comma)',
    callback=parse_show_only,
)
def live_simple(interval, show_only):
    old_collections = (None, None, None)
    with Live(auto_refresh=False) as live:
        while True:
            collections = get_prices(show_only)

            prices_text = Text()
            prices_text.append(f'{datetime.now().strftime("%Y-%m-%d %H:%M:%S")}\n\n', style='bold')

            for collection_index, collection in enumerate(collections):
                for index, model in enumerate(collection):
                    old_collection = old_collections[collection_index]
                    if old_collection is not None \
                            and len(old_collection) > index \
                            and old_collection[index].code == model.code:
                        old_model = old_collection[index]
                    else:
                        old_model = None

                    prices_text.append(model.assemble_simple_text(old_model))

            prices_text.rstrip()
            live.update(prices_text, refresh=True)
            old_collections = collections
            time.sleep(interval)


# ================ bonbast live currency ================
# todo: Make sure that the table scroll when updating. Also we can add the newest one to the top.
# todo: research on how to add delta of the price change (if done, also add to "live simple" command)
@live.command('currency')
@click.option('-i', '--interval', type=click.IntRange(min=1), default=30, help='Interval in seconds')
@click.argument(
    'currency', nargs=1,
    type=click.Choice(
        list(Currency.VALUES.keys()) + list(Gold.VALUES.keys()) + list(Coin.VALUES.keys()), case_sensitive=False
    )
)
def live_currency(interval, currency):
    table = Table()
    first_time = True
    old_model = None
    with Live(table, auto_refresh=False) as live:
        while True:
            # request to get all the currencies
            collections = get_prices([currency])

            # flatten the list of currencies and get the first element
            items = (item for collection in collections for item in collection)
            item = next(items)

            # if it's the first time, add the header
            if first_time:
                table.add_column("Date", no_wrap=True)
                table.add_column("Name", no_wrap=True)
                if type(item) is Currency or type(item) is Coin:
                    table.add_column("Sell", no_wrap=True)
                    table.add_column("Buy", no_wrap=True)
                else:
                    table.add_column("Price", no_wrap=True)

                first_time = False

            # add a row with the new information
            if type(item) is Currency or type(item) is Coin:
                sell_symbol = f' {get_change_char(item.sell, old_model.sell)}' if old_model is not None else ''
                sell_str = (item.formatted_sell if item.sell is not None else '-') + sell_symbol
                sell_color = get_color(item.sell, None if old_model is None else old_model.sell)

                buy_symbol = f' {get_change_char(item.buy, old_model.buy)}' if old_model is not None else ''
                buy_str = (item.formatted_buy if item.buy is not None else '-') + buy_symbol
                buy_color = get_color(item.buy, None if old_model is None else old_model.buy)

                price = (
                    Text(sell_str, style=sell_color),
                    Text(buy_str, style=buy_color),
                )
            else:
                price_char = f' {get_change_char(item.price, old_model.price)}' if old_model is not None else ''
                price_str = (item.formatted_price if item.price is not None else '-') + price_char
                price_color = get_color(item.price, None if old_model is None else old_model.price)

                price = (
                    Text(price_str, style=price_color),
                )

            table.add_row(
                Text(datetime.now().strftime("%Y-%m-%d %H:%M:%S"), style=DEFAULT_TEXT_COLOR),
                Text(item.name, style=DEFAULT_TEXT_COLOR),
                *price,
            )

            live.update(table, refresh=True)
            old_model = item
            time.sleep(interval)


# ================ bonbast convert ================
@cli.command()
@click.option('-s', '--source', type=click.Choice(Currency.VALUES, case_sensitive=False))
@click.option('-d', '--destination', type=click.Choice(Currency.VALUES, case_sensitive=False))
@click.argument('amount', type=click.FLOAT)
@click.option('--only-buy', is_flag=True, default=False, help='Only show buy price')
@click.option('--only-sell', is_flag=True, default=False, help='Only show sell price')
@click.pass_context
def convert(ctx, source, destination, amount, only_buy, only_sell):
    if source is None and destination is None:
        raise click.BadOptionUsage('', 'Please specify source or destination currency')

    if source is not None and destination is not None:
        raise click.BadOptionUsage('', 'Don\'t use --source and --destination together')

    if only_buy and only_sell:
        raise click.BadOptionUsage('', 'Don\'t use --only-buy and --only-sell together')

    currencies_list, _, __ = get_prices()
    if source is not None:
        currency = next(currency for currency in currencies_list if currency.code.lower() == source.lower())
        buy = format_toman(int(amount * currency.buy))
        sell = format_toman(int(amount * currency.sell))
    else:
        currency = next(currency for currency in currencies_list if currency.code.lower() == destination.lower())
        buy = format_price(amount / currency.sell)
        sell = format_price(amount / currency.buy)

    if only_buy:
        click.echo(buy)
    elif only_sell:
        click.echo(sell)
    else:
        click.echo(f'{buy} / {sell}')


# ================ bonbast history ================
@cli.command()
@click.option(
    '--date',
    type=click.DateTime(formats=["%Y/%m/%d", '%Y-%m-%d']),
    prompt=True,
    help='Date to get prices for'
)
def history(date):
    currencies = get_history(date)
    console = Console()
    console.print(get_currencies_table(currencies, 2))


# ================ bonbast export ================
@cli.command()
@click.option('--pretty', is_flag=True, default=False, help='Pretty print the output')
@click.option('--expanded', is_flag=True, default=False, help='Tries to expand the JSON')
@click.option(
    '--show-only',
    help='Show only specified currencies, coins, or golds (separated by comma)',
    callback=parse_show_only,
)
def export(pretty, expanded, show_only):
    items = get_prices(show_only)
    prices = {}
    for item in items:
        for model in item:
            prices.update(model.to_json())

    if pretty:
        pprint(prices, expand_all=expanded)
    else:
        click.echo(json.dumps(prices, ensure_ascii=False))


if __name__ == '__main__':
    cli()



import re
from datetime import datetime, timedelta
from typing import Tuple, List, Optional, Dict

import requests
from bs4 import BeautifulSoup

try:
    from .models import *
except ImportError:
    from models import *

BASE_URL = 'https://www.bonbast.com'
USER_AGENT = 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) ' \
             'Chrome/103.0.0.0 Mobile Safari/537.36'
SELL = '1'
BUY = '2'


def int_try_parse(value) -> Optional[int]:
    try:
        return int(value)
    except ValueError:
        return None


def get_token_from_main_page():
    """ This function gets a token from main page of bonbast.com
    This token should be saved somewhere because we are going to need it when we want to load pricest

    param dont_raise_error: It's on False by default. If you pass True it won't raise error because of connection problems.
        It will return False instead.
    """
    headers = {
        'authority': 'bonbast.com',
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-language': 'en-US,en;q=0.9,fa;q=0.8',
        'cache-control': 'no-cache',
        'cookie': 'cookieconsent_status=true; st_bb=0',
        'pragma': 'no-cache',
        'sec-ch-ua': '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
        'sec-ch-ua-mobile': '?1',
        'sec-ch-ua-platform': '"Android"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'none',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': USER_AGENT,
    }

    try:
        r = requests.get(BASE_URL, headers=headers)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise SystemExit(err)

    search = re.search(r"param\s*=\s*\"(.+)\"", r.text, re.MULTILINE)
    if search is None or search.group(1) is None:
        raise SystemExit('Error: token not found in the main page')

    return search.group(1)


def get_prices_from_api(token: str) -> Tuple[List[Currency], List[Coin], List[Gold]]:
    """ Gets the prices' data from API using

    param token: You should pass the token that you got from get_token_from_main_page

    param dont_raise_error: It's on False by default. If you pass True it won't raise error because of connection problems.
        It will return False instead.
    """
    headers = {
        'authority': 'bonbast.com',
        'accept': 'application/json, text/javascript, */*; q=0.01',
        'accept-language': 'en-US,en;q=0.9,fa;q=0.8',
        'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
        'cookie': 'cookieconsent_status=true; st_bb=0',
        'origin': 'https://bonbast.com',
        'referer': 'https://bonbast.com/',
        'sec-ch-ua': '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
        'sec-ch-ua-mobile': '?1',
        'sec-ch-ua-platform': '"Android"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': USER_AGENT,
        'x-requested-with': 'XMLHttpRequest',
    }

    data = {
        'data': token,
        'webdriver': 'false',
    }

    try:
        r = requests.post(f'{BASE_URL}/json', headers=headers, data=data)
        r.raise_for_status()
        r = r.json()
    except requests.exceptions.HTTPError as err:
        raise SystemExit(err)

    if 'reset' in r:
        raise ResetAPIError('Error: token is expired')

    currencies: List[Currency] = []
    coins: List[Coin] = []
    golds: List[Gold] = []

    for currency in Currency.VALUES:
        if f'{currency}{BUY}' in r and f'{currency}{SELL}' in r:
            currencies.append(Currency(
                currency.upper(),
                Currency.VALUES[currency],
                sell=int(r[f'{currency}{SELL}']),
                buy=int(r[f'{currency}{BUY}']),
            ))

    for coin in Coin.VALUES:
        if f'{coin}' in r and f'{coin}{BUY}' in r:
            coins.append(Coin(
                coin,
                Coin.VALUES[coin],
                sell=int(r[coin]),
                buy=int(r[f'{coin}{BUY}']),
            ))

    for gold in Gold.VALUES:
        if f'{gold}' in r:
            golds.append(Gold(
                gold,
                Gold.VALUES[gold],
                price=int(r[gold])
            ))

    return currencies, coins, golds


def get_graph_data(
        currency: str,
        start_date: datetime = datetime.today() - timedelta(days=30),
        end_date: datetime = datetime.today(),
) -> Dict[str, int]:
    """
        This function will make a request to bonbast.com/graph and make them in two array.
    """

    headers = {
        'authority': 'bonbast.com',
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-language': 'en-US,en;q=0.9,fa;q=0.8',
        'cache-control': 'max-age=0',
        'cookie': 'cookieconsent_status=true; st_bb=0',
        'referer': 'https://bonbast.com/',
        'sec-ch-ua': '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
        'sec-ch-ua-mobile': '?1',
        'sec-ch-ua-platform': '"Android"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'same-origin',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': USER_AGENT,
    }

    try:
        request = requests.get(f'{BASE_URL}/graph/{currency}/{start_date.date()}/{end_date.date()}', headers=headers)
        request.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise SystemExit(err)

    soup = BeautifulSoup(request.text, 'html.parser')
    for data in soup.find_all("script"):
        # get variables from script and convert them to list
        if "data: {" in data.text:
            price_list = data.text.split("data: [")[1].split("]")[0].split(",")
            date_list = data.text.split("labels: [")[1].split("]")[0].split(',')

            if len(price_list) != len(date_list):
                raise SystemExit('Error: data inconsistency')

            dic = {}
            for i in range(len(price_list)):
                price = int(price_list[i])
                date = re.search(r'\d{4}-(0[1-9]|1[0-2])-(0[1-9]|[12]\d|3[01])', date_list[i]).group(0)
                dic[date] = price

            return dic


def get_history(date: datetime = datetime.today() - timedelta(days=1)) -> List[Currency]:
    if date.date() < datetime(2012, 10, 9).date():
        raise SystemExit('Error: date is too far in the past. Date must be greater than 2012-10-09')

    if date.date() >= datetime.today().date():
        raise SystemExit(f'Error: date must be less than today({date.today().date()}).')

    headers = {
        'authority': 'bonbast.com',
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
        'accept-language': 'en-US,en;q=0.9,fa;q=0.8',
        'cache-control': 'max-age=0',
        'sec-ch-ua': '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
        'sec-ch-ua-mobile': '?1',
        'sec-ch-ua-platform': '"Android"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'same-origin',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': USER_AGENT,
    }

    try:
        request = requests.get(f'{BASE_URL}/archive/{date.strftime("%Y/%m/%d")}', headers=headers)
        request.raise_for_status()
    except requests.exceptions.HTTPError as err:
        raise SystemExit(err)

    soup = BeautifulSoup(request.text, 'html.parser')
    tables = soup.findAll("table")

    # first and second table are currencies
    currencies: List[Currency] = []
    for table in tables[0:1]:
        for row in table.findAll('tr')[1:]:
            cells = row.findAll("td")
            currencies.append(Currency(
                cells[0].text.lower(),
                Currency.VALUES[cells[0].text.lower()],
                sell=int_try_parse(cells[2].text),
                buy=int_try_parse(cells[3].text),
            ))

    # todo: parse it correctly
    # last table is coins
    # coins: List[Coin] = []
    # for table in tables[-1:]:
    #     for row in table.findAll('tr')[1:]:
    #         cells = row.findAll("td")
    #         coins.append(Coin(
    #             cells[0].text.lower(),
    #             Coin.VALUES[cells[0].text.lower()],
    #             sell=int_try_parse(cells[2].text),
    #             buy=int_try_parse(cells[3].text),
    #         ))

    return currencies


class ResetAPIError(Exception):
    """
    This exception is raised when the token is expired, and you have to get new one.
    """
    pass



try:
    from ..helpers.utils import *
except ImportError:
    from src.bonbast.helpers.utils import *


class Graph:
    """ Graph model
    """

    def __init__(self, date: str, value: int, currency: str):
        self.date = date
        self.value = value
        self.currency = currency

    def to_json(self) -> dict:
        return {
            self.currency: {
                'date': self.date,
                'value': self.value,
            }
        }



import click

try:
    from ..__init__ import *
except ImportError:
    from src.bonbast.__init__ import *


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    else:
        click.echo(bonbast_version)
        ctx.exit()


def parse_show_only(ctx, param, value):
    if value is not None and value != '':
        if value[0] == '\'' and value[-1] == '\'':
            value = value[1:-1]
        value = value.split(',')
        return [item.lower().strip() for item in value]



import os
import pathlib
import sys

try:
    from ..helpers.utils import Singleton
except:  # noqa
    from src.bonbast.helpers.utils import Singleton


class StorageManager(object):
    def __init__(self, file_path: pathlib.Path):
        self.storage_path = StorageManager.get_app_directory()
        self.file_path = file_path

    @staticmethod
    def get_app_directory() -> pathlib.Path:
        """ Returns a directory path
        data directory is where persistent application data can be stored.
        - linux: ~/.local/share
        - macOS: ~/Library/Application Support
        - windows: C:/Users/<USER>/AppData/Roaming

        the final directory is where application data can be stored.
        """

        home = pathlib.Path.home()

        if sys.platform == "win32":
            data_dir = home / "AppData/Roaming"
        elif sys.platform == "linux":
            data_dir = home / ".local/share"
        elif sys.platform == "darwin":
            data_dir = home / "Library/Application Support"
        else:
            raise NotImplementedError(f"Platform {sys.platform} is not supported")

        return data_dir / "bonbast"

    def save_file(self, content: str) -> None:
        """ Gets token it in data directory
        """
        self.storage_path.mkdir(parents=True, exist_ok=True)

        with open(self.file_path, 'w') as f:
            f.write(content)

    def load_file(self) -> str:
        """ Loads the saved token from datadir.
        Returns token, datetime
        Raise FileNotFoundError if no token was founded
        """
        with open(self.file_path, 'r') as f:
            return f.read()

    def delete_file(self) -> None:
        """ Delete the saved token from datadir
        """
        try:
            os.remove(self.file_path)
        except FileNotFoundError:
            pass



import time
import functools

TOMAN_FORMATTER = "{:,}"
PRICE_FORMATTER = "{:,.2f}"

DEFAULT_TEXT_COLOR = 'cyan'


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


def format_toman(price: float) -> str:
    return TOMAN_FORMATTER.format(price)


def format_price(price: float) -> str:
    return PRICE_FORMATTER.format(price)


def retry(retry_count=3, retry_delay=None, message=''):
    def decorator_retry(func):
        @functools.wraps(func)
        def wrapper_retry(*args, **kwargs):
            for _ in range(retry_count):
                try:
                    return func(*args, **kwargs)
                except:  # noqa
                    if retry_delay is not None:
                        time.sleep(retry_delay)

            raise SystemExit(message)

        return wrapper_retry

    return decorator_retry


def get_color(price, old_price):
    if old_price is None or price is None:
        return ''

    if price > old_price:
        return 'green'
    elif price < old_price:
        return 'red'
    else:
        return ''


def get_change_char(price, old_price):
    if old_price is None or price is None:
        return ''

    if price > old_price:
        return '↑'
    elif price < old_price:
        return '↓'
    else:
        return '-'



from __future__ import annotations

from rich.text import Text

try:
    from ..helpers.utils import *
except ImportError:
    from src.bonbast.helpers.utils import *


class Gold:
    """ Gold model
    """
    VALUES = {
        'mithqal': 'Gold Mithqal',
        'gol18': 'Gold Gram',
    }

    def __init__(self, code: str, name: str, price: float):
        self.code = code
        self.name = name
        self.price = price

    @property
    def formatted_price(self) -> str:
        return format_toman(self.price)

    def to_json(self) -> dict:
        return {
            self.code: {
                'name': self.name,
                'price': self.price,
            }
        }

    def assemble_simple_text(self, old_gold: Gold, **kwargs) -> Text:
        return Text.assemble(
            f'{self.code}: ',
            (f'{self.price}', get_color(self.price, old_gold.price) if old_gold is not None else ''),
            '\n'
        )



from __future__ import annotations

from rich.text import Text

try:
    from ..helpers.utils import *
    from ..helpers.currency_flags import *
except ImportError:
    from src.bonbast.helpers.utils import *
    from src.bonbast.helpers.currency_flags import *


class Currency:
    """ Currency model
    """
    VALUES = {
        'usd': 'US Dollar',
        'eur': 'Euro',
        'gbp': 'British Pound',
        'chf': 'Swiss Franc',
        'cad': 'Canadian Dollar',
        'aud': 'Australian Dollar',
        'sek': 'Swedish Krona',
        'nok': 'Norwegian Krone',
        'rub': 'Russian Ruble',
        'thb': 'Thai Baht',
        'sgd': 'Singapore Dollar',
        'hkd': 'Hong Kong Dollar',
        'azn': 'Azerbaijani Manat',
        'amd': '10 Armenian Dram',
        'dkk': 'Danish Krone',
        'aed': 'UAE Dirham',
        'jpy': '10 Japanese Yen',
        'try': 'Turkish Lira',
        'cny': 'Chinese Yuan',
        'sar': 'Saudi Riyal',
        'inr': 'Indian Rupee',
        'myr': 'Malaysian Ringgit',
        'afn': 'Afghan Afghani',
        'kwd': 'Kuwaiti Dinar',
        'iqd': '100 Iraqi Dinar',
        'bhd': 'Bahraini Dinar',
        'omr': 'Omani Rial',
        'qar': 'Qatari Rial',
    }

    def __init__(self, code: str, name: str, sell: int, buy: int):
        self.code = code
        self.name = name
        self.sell = sell
        self.buy = buy

    @property
    def formatted_buy(self) -> str:
        return format_toman(self.buy)

    @property
    def formatted_sell(self) -> str:
        return format_toman(self.sell)

    @property
    def flag(self) -> str:
        return currency_flags[self.code.lower()]

    def to_json(self) -> dict:
        return {
            self.code: {
                'name': self.name,
                'sell': self.sell,
                'buy': self.buy,
            }
        }

    def assemble_simple_text(self, old_currency: Currency, with_flag: bool = False) -> Text:
        return Text.assemble(
            (f'{self.flag} ' if with_flag else '') + f'{self.code}: ',
            (f'{self.formatted_sell}', get_color(self.sell, old_currency.sell) if old_currency is not None else ''),
            ' / ',
            (f'{self.formatted_buy}', get_color(self.buy, old_currency.buy) if old_currency is not None else ''),
            '\n'
        )
